<?php

namespace Drupal\page_manager_search\Language;

use Drupal\Core\Language\LanguageDefault;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\WriteSafeSessionHandlerInterface;

/**
 * An implementation of LanguageSwitcherInterface.
 *
 * This allows for safe switching of language by ensuring that session
 * data for one language is not leaked in to others.
 * It also provides a stack that allows reverting to a previous user after
 * switching.
 */
class LanguageSwitcher implements LanguageSwitcherInterface {

  /**
   * A stack of previous overridden languages.
   *
   * @var \Drupal\core\Language\LanguageInterface[]
   */
  protected $languageStack = [];

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The default language.
   *
   * @var \Drupal\Core\Language\LanguageDefault
   */
  protected $languageDefault;

  /**
   * The write-safe session handler.
   *
   * @var \Drupal\Core\Session\WriteSafeSessionHandlerInterface
   */
  protected $writeSafeHandler;

  /**
   * The original state of session saving prior to language switching.
   *
   * @var bool
   */
  protected $originalSessionSaving;

  /**
   * Constructs a new LanguageSwitcher.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Language\LanguageDefault $language_default
   *   The default language.
   * @param \Drupal\Core\Session\WriteSafeSessionHandlerInterface $write_safe_handler
   *   The write-safe session handler.
   */
  public function __construct(LanguageManagerInterface $language_manager, LanguageDefault $language_default, WriteSafeSessionHandlerInterface $write_safe_handler) {
    $this->languageManager = $language_manager;
    $this->languageDefault = $language_default;
    $this->writeSafeHandler = $write_safe_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function switchTo(LanguageInterface $language) {
    // Prevent session information from being saved and push previous account.
    if (!isset($this->originalSessionSaving)) {
      // Ensure that only the first session saving status is saved.
      $this->originalSessionSaving = $this->writeSafeHandler->isSessionWritable();
    }
    $this->writeSafeHandler->setSessionWritable(FALSE);
    array_push($this->languageStack, $this->languageDefault->get());
    $this->languageDefault->set($language);
    $this->languageManager->reset();
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function switchBack() {
    if (empty($this->languageStack)) {
      throw new \RuntimeException('No more languages to revert to.');
    }

    // Restore the previous language from the stack.
    $this->languageDefault->set(array_pop($this->languageStack));
    $this->languageManager->reset();

    // Restore original session saving status if all language switches are
    // reverted.
    if (empty($this->languageStack) && $this->originalSessionSaving) {
      $this->writeSafeHandler->setSessionWritable(TRUE);
    }
    return $this;
  }

}
